@@ -209,15 +209,12 @@ on_tools_data_sources (GtkAction *action, WorkspaceWindowPrivate *priv)
 
 static void console_destroyed_cb (GtkWidget *widget, WorkspaceWindowPrivate *priv);
 static void
-on_sql_console (GtkAction *action, void *data)
+on_sql_console (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
 	GtkWidget *dlg, *sw, *console;
-	WorkspaceWindowPrivate *priv;
 	gchar *title;
 	GdaConnection *cnc;
 
-	priv = g_object_get_data (G_OBJECT (data), "Mergeant_WorkspaceWindowPrivate");
-
 	/* dialog */
 	dlg = gtk_dialog_new_with_buttons (_("SQL console"), NULL, 0,
 					   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
@@ -349,7 +346,7 @@ static GtkActionEntry ui_actions[] = {
 	{ "DatabaseCreate", GTK_STOCK_NEW, "_Create a database", NULL, "Create a new database", G_CALLBACK (on_database_new) },
 	{ "DatabaseDelete", GTK_STOCK_DELETE, "_Delete a database", NULL, "Delete an existing database", G_CALLBACK (on_database_del) },
 	{ "ToolsDataSources", GTK_STOCK_PREFERENCES, "_Datasources", NULL, "Manage the list of declared datasources", G_CALLBACK (on_tools_data_sources) },
-	{ "SqlConsole", NULL, "_Console SQL", NULL, "Open an SQL console", G_CALLBACK (on_sql_console) },
+	{ "SqlConsole", GNOME_DB_STOCK_SQL_CONSOLE, "_Console SQL", NULL, "Open an SQL console", G_CALLBACK (on_sql_console) },
 	{ "About", NULL, "_About", NULL, "About", NULL },
 	{ "HelpAbout", GTK_STOCK_ABOUT, "_About", NULL, "Show information about Mergeant", G_CALLBACK (on_help_about) }
 };
@@ -385,6 +382,7 @@ static const gchar *ui_actions_info =
         "  </menubar>"
         "  <toolbar  name='ToolBar'>"
         "    <toolitem action='ToolsDataSources'/>"
+        "    <toolitem action='SqlConsole'/>"
 	"    <separator/>"
         "  </toolbar>"
         "</ui>";
@@ -393,7 +391,7 @@ static const gchar *ui_actions_info =
 static void
 stop_dbms_update_cb (GtkWidget *dialog, gint response, GdaDict *dict)
 {
-	gda_dict_stop_update_dbms_data (dict);
+	gda_dict_stop_update_dbms_meta_data (dict);
 }
 
 static void
@@ -414,7 +412,7 @@ perform_dbms_update (WorkspaceWindowPrivate *priv)
 	gtk_widget_show (dialog);
 	
 	error = NULL;
-	if (!gda_dict_update_dbms_data (priv->dict, 0, NULL, &error)) {
+	if (!gda_dict_update_dbms_meta_data (priv->dict, 0, NULL, &error)) {
 		if (error->code != GDA_DICT_META_DATA_UPDATE_USER_STOPPED) {
 			GtkWidget *msg;
 
@@ -680,8 +678,10 @@ thread_open_connect_finished (GdaThreader *thread, guint job_id, ThreadData *thd
 		gtk_widget_destroy (msg);
 
 		/* gdk_threads_leave (); */
-		if (error)
+		if (error) {
 			g_error_free (error);
+			*(thdata->error) = NULL;
+		}
 		quit_if_no_other_workspace ();
 	}
 	else {
@@ -852,7 +852,7 @@ create_new_workspace_window_no_dict_choose_cb (GtkDialog *dialog, gint response,
 		GtkWidget *wid;		
 		gchar *str;
 
-		workdict = (GdaDict *) gda_dict_new ();
+		workdict = gda_dict_new ();
 		
 		filename = gda_dict_compute_xml_filename (workdict, dsn, NULL, NULL);
 		gda_dict_set_xml_filename (workdict, filename);
